#!/usr/bin/perl -w
#
#############################################################################
# cdlabel-latex v0.1
#
# Create a LaTeX CD Label from an amarok playlist
# see http://www.tug.org/tex-archive/macros/latex/contrib/cd-cover/
#
# Carsten Jahn		carsten_jahn@web.de
# (c) 2005
#
############################################################################
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
############################################################################
# tabsize = 4

# TODO: remember last settings

use warnings;
use strict;

package MainWindow;

use Qt;
use Qt::isa qw(Qt::Widget);
use Qt::slots
	refreshPlaylistInfo => [],
	templateFilenameButClicked => [],
	updateMediaTitles => [],
	createLabelClicked => [],
	quitClicked => [],
	displayLogClicked => [],
	displayTexfileClicked => [];
use Cwd;  # current working directory
use File::stat; # by-name interface to Perl's built-in stat functions
 
# Definition of Qt Widgets
my $playlistInfo;
my $playlistInfoRefreshBut;
my $templateFilename;
my $templateFilenameBut;
my $symbolPreviewList;
my $mediaTitle;
my $mediaSubtitle;


sub NEW {
	shift->SUPER::NEW(@_);
	this->setName('mainWindow');
	
	read_iso_table();
	
	#
	# Construct GUI elements
	#
	$playlistInfo = Qt::Label( this, 'playlistInfo' );
	$playlistInfo->setText( "..." );
	$playlistInfoRefreshBut = Qt::PushButton( "Fetch Playlist", this, 'playlistInfoRefreshBut' );
	this->connect($playlistInfoRefreshBut, SIGNAL('clicked()'), SLOT('refreshPlaylistInfo()'));
	
	$templateFilename = Qt::LineEdit( this, 'templateFilename' );
	$templateFilenameBut = Qt::PushButton( 'Select', this, 'templateFilenameBut' );
	this->connect($templateFilenameBut, SIGNAL 'clicked()', SLOT 'templateFilenameButClicked()');
	
	$mediaTitle = Qt::LineEdit( this, 'mediaTitle' );
	this->connect($mediaTitle, SIGNAL 'textChanged(const QString &)', SLOT 'updateMediaTitles()');
	$mediaSubtitle = Qt::LineEdit( this, 'mediaSubtitle' );
	this->connect($mediaSubtitle, SIGNAL 'textChanged(const QString &)', SLOT 'updateMediaTitles()');
	
	$symbolPreviewList = Qt::ListView( this, 'symbolPreviewList' );
	$symbolPreviewList->addColumn( 'Symbol' );
	$symbolPreviewList->addColumn( 'Current Text' );
	$symbolPreviewList->setSelectionMode( Qt::ListView::NoSelection() );
	#$symbolPreviewList->setFixedSize( 450, 200); works, but not resizeable even if user resizes window
	
	my $createLabelBut = Qt::PushButton( 'Create Label', this, 'createLabelBut' );
	this->connect($createLabelBut, SIGNAL 'clicked()', SLOT 'createLabelClicked()');
	my $displayLogBut = Qt::PushButton( 'View LaTeX Logfile', this, 'displayLogBut' );
	this->connect($displayLogBut, SIGNAL 'clicked()', SLOT 'displayLogClicked()');
	my $displayTexfileBut = Qt::PushButton( 'View Generated TeX File', this, 'displayTexfileBut' );
	this->connect($displayTexfileBut, SIGNAL 'clicked()', SLOT 'displayTexfileClicked()');
	my $quitBut = Qt::PushButton( 'Quit', this, 'quitBut' );
	this->connect($quitBut, SIGNAL 'clicked()', SLOT 'quitClicked()');

	
	#
	# arrange GUI layout
	#
	my $grid = Qt::GridLayout(this, 6, 2, 10);  # height, width (# cells)
	my $currentrow = 0;
	
	my $box = Qt::HBoxLayout;
	# the variable $box holds all "sublayouts" that are constructed and then added to the grid
	$box->addWidget( $playlistInfo );
	$box->addWidget( $playlistInfoRefreshBut );
	$grid->addWidget(Qt::Label('amaroK Playlist',this), $currentrow,0 );
	$grid->addLayout($box, $currentrow++,1 );
	Qt::ToolTip::add($playlistInfoRefreshBut,"This loads the current amaroK playlist\ninto the playlist-cdcover tool.");
	
	$box = Qt::HBoxLayout;
	$box->addWidget( $templateFilename );
	$box->addWidget( $templateFilenameBut );
	$grid->addWidget(Qt::Label('TeX Template File',this), $currentrow,0 );
	$grid->addLayout($box, $currentrow++,1 );
	Qt::ToolTip::add( $templateFilename,
	".tex files with special 'placeholders' (variables) are used. Select one of the supplied\n" .
	"templates or build your own. The selected templates are responsible for formatting\n" .
	"the information found in your current playlist.\n" .
	"The templates define which type of information (titles, only artist names, ...)\n" .
	"will be present in the output document.\n\n" . 
	"You'll find the supplied templates in ~/.kde/share/apps/amarok/scripts/amarok-playlist-cdcover/...");
	Qt::ToolTip::add( $templateFilenameBut,
	".tex files with special 'placeholders' (variables) are used. Select one of the supplied\n" .
	"templates or build your own. The selected templates are responsible for formatting\n" .
	"the information found in your current playlist.\n" .
	"The templates define which type of information (titles, only artist names, ...)\n" .
	"will be present in the output document.\n\n" . 
	"You'll find the supplied templates in ~/.kde/share/apps/amarok/scripts/amarok-playlist-cdcover/...");
	
	$grid->addWidget(Qt::Label("Media Title",this), $currentrow,0 );
	$grid->addWidget($mediaTitle, $currentrow++,1 );
	Qt::ToolTip::add($mediaTitle, "Some of the templates use this variable, others don't.\n" .
	"For example, \"One Artist Album.tex\" does not need any additional information\n" .
	"because everything is in the playlist. Other templates are designed for whole collections,\n" .
	"i.e. multiple albums, and require the user to enter text for the front page.");

	$grid->addWidget(Qt::Label("Media Subtitle",this), $currentrow,0 );
	$grid->addWidget($mediaSubtitle, $currentrow++,1 );
	Qt::ToolTip::add($mediaSubtitle,"See the Media Title tooltip.");

	$grid->addWidget(Qt::Label("Variable Preview",this), $currentrow,0 );
	$grid->addWidget($symbolPreviewList, $currentrow++,1 );
	Qt::ToolTip::add($symbolPreviewList,
	"This list shows you the variables and their current definition.\n" .
	"Use this to preview which substitutions will be made during the CD cover creation.\n" .
	"If you do not design your own templates, you'll probably don't need these details." );
	
	$box = Qt::HBoxLayout;
	$box->addWidget( $createLabelBut );
	Qt::ToolTip::add($createLabelBut,
	"This reads the template and writes a temporary TeX file with\n" .
	"variables substituted as shown above. TeX is started. The TeX dvi output\n" .
	"is converted to postscript with dvips and displayed with kghostview.\n" .
	"See README for requirements." );
	$box->addWidget( $displayLogBut );
	Qt::ToolTip::add($displayLogBut, "Displays TeX's logfile with kwrite.");
	$box->addWidget( $displayTexfileBut );
	Qt::ToolTip::add($displayTexfileBut, "Displays the generated .tex input file with kwrite.");
	$box->addWidget( $quitBut );
	$grid->addMultiCellLayout($box, $currentrow,$currentrow,0,1, Qt::AlignRight() );
	$currentrow++;
	
	
	# do initial playlist loading
	refreshPlaylistInfo();
	
	# all to no avail... 
	# TODO: smaller size in gui
	#$symbolPreviewList->resize(50,50);
	$symbolPreviewList->resizeContents(50,50);
	#updateGeometry();
}


my $playlist; # last read playlist
sub refreshPlaylistInfo()
{
	# This also reads the playlist. startClicked() is based on the playlist information.

	############################################################################
	#
	# Configuration
	#
	
	my $path2currentxml = "/.kde/share/apps/amarok/current.xml";
	
	############################################################################
	my $dialog1 = "";
	my $dialog2 = "";
	
	# if amarok is running ->
	# refresh current.xml
	my $amarokrunning = `dcop amarok 2> /dev/null`;
	chomp($amarokrunning);
	if (!($amarokrunning =~ s/^$/No such application: 'amarok'/)) {
		system "dcop amarok playlist saveCurrentPlaylist";
	}
	# read XML file
	$playlist = XMLin($ENV{HOME} . $path2currentxml);
	
	# count distinct artist, albums and number of songs for informational purposes.
	my $song_count = 0;
	my %artists; # keys are artist names to count automatically distinct artists
	my %artist_albums; # key inserted for every combination of artist and album
	foreach my $plitem (@{$playlist->{item}}) {
		$artists{ $plitem->{Artist} } = 1;
		$artist_albums{ $plitem->{Artist} . '_' . $plitem->{Album} } = 1;
		$song_count++;
	}
	$playlistInfo->setText( scalar keys(%artists) . " Artists, " .
		scalar keys(%artist_albums) . " Albums, " .
		$song_count . " Songs" );
	
	rebuildSymbolDefs();
}


sub templateFilenameButClicked
{
	my $returnfile = Qt::FileDialog::getOpenFileName(
                    ".",
                    'TeX Templates (*.tex)',
                    this,
                    "open file dialog",
                    "Choose a .tex-template file" );
	if( length($returnfile) > 0 ) {
		$templateFilename->setText($returnfile);
	}
}

sub quitClicked
{
	# TODO remove tmp file
	Qt::app()->quit();
}


use XML::Simple;
use File::Copy;
use File::Basename;
use File::Path;
use Filesys::Df;


sub texString
{
	my $str = $_[0];
	if( ! defined( $str ) ) {
		return "";
	}
	# trim leading / trailing blanks
	$str =~ s/\s+$//m;
    $str =~ s/^\s+//m;
	# handle accents, umlauts etc
	$str = iso2tex( $str );
	
	return $str;
}

my %symbolDefs;

sub rebuildSymbolDefs
{
	# ---------  pre-define template variables for later use in template copy loop:
	# *** AllTitleCalls ***
	my $allTitleCalls = '';
	my $sequentialNumber = 1;
	foreach my $plitem (@{$playlist->{item}}) {
		if( $sequentialNumber > 1 ) {
			$allTitleCalls = $allTitleCalls .  "\\ElementDelimiter\n";
		}
		#1 SequentialNumber
		#2 Artist
		#3 Album
		#4 AlbumTrackno
		#5 Title
		#6 Year
		#7 Length
		#8 Bitrate
		$allTitleCalls = $allTitleCalls . sprintf("\\Title{%d}{%s}{%s}{%d}{%s}{%s}{%s}{%s}\n",
			$sequentialNumber, texString($plitem->{Artist}), texString($plitem->{Album}),
			texString($plitem->{TrackNo}), texString($plitem->{Title}),
			texString($plitem->{Year}), texString($plitem->{Length}), texString($plitem->{Bitrate}) );
		
		$sequentialNumber = $sequentialNumber + 1;
	}
	chomp($allTitleCalls);

	
	# *** allArtistAlbumCalls ***
	# a hash map of artists is created, where the values are hashmaps of album names of that artist.
	my %artists_albums;
	my $albums_ref;
	foreach my $plitem (@{$playlist->{item}}) {
		if( defined($artists_albums{ $plitem->{Artist} } ) ) {
			# add album to existing artist
			# my %albums;
			$albums_ref = $artists_albums{ $plitem->{Artist} };
			$$albums_ref{ $plitem->{Album} } = 1; # add key, duplicate albums are omitted automatically here.
			#$albumlist_reference = $artists_albums{ $plitem->{Artist} };
			#push( @$albumlist_reference, $plitem->{Album} );
			#print "added album to artist: " . $plitem->{Artist} . "->" . @$albumlist_reference . "\n";
		} else {
			# create new album hash with one element
			my %albums;
			$albums{ $plitem->{Album} } = 1;
			$artists_albums{ $plitem->{Artist} } = \%albums;
			#  \ creates a reference to %albums, enabling storage of albums hashmap in artists_albums hashmap.
		}
	}
	my $allArtistAlbumCalls = '';
	my $artist;
	my @artists_sorted = sort( keys(%artists_albums) );
	# loop over artists
	foreach $artist (@artists_sorted) {
		# get hash of albums of this artist
		$albums_ref = $artists_albums{$artist};
		my @albums_sorted = sort( keys(%$albums_ref) );
		my $album;
		foreach $album (@albums_sorted) {
			#print $artist . " -- " . $album . "\n";
			if( length($allArtistAlbumCalls) > 0 ) {
				$allArtistAlbumCalls = $allArtistAlbumCalls . "\\ElementDelimiter\n";
			}
			#1 Artist
			#2 Album
			$allArtistAlbumCalls = $allArtistAlbumCalls . sprintf("\\ArtistAlbum{%s}{%s}\n",
				texString($artist), texString($album) );
		}
	}
	chomp($allArtistAlbumCalls);

	# *** allArtistCalls ***
	# reuse array created above for allArtistAlbumCalls.
	my $allArtistCalls = '';
	foreach $artist (@artists_sorted) {
		if( length($allArtistCalls) > 0 ) {
			$allArtistCalls = $allArtistCalls . "\\ElementDelimiter\n";
		}
		#1 Artist
		$allArtistCalls = $allArtistCalls . sprintf("\\Artist{%s}\n", texString($artist) );
	}
	
	# *** Name all variables ***
	# if preceded with $, these variables will be automatically substituted with the strings
	# that were built above.
		
	my $firstPlaylistItem = $playlist->{item}->[0];
	
	$symbolDefs{FirstArtist} = texString($firstPlaylistItem->{Artist});
	$symbolDefs{FirstAlbum} = texString($firstPlaylistItem->{Album});
	$symbolDefs{FirstYear} = texString($firstPlaylistItem->{Year});
	$symbolDefs{AllTitleCalls} = $allTitleCalls;
	$symbolDefs{AllArtistAlbumCalls} = $allArtistAlbumCalls;
	$symbolDefs{AllArtistCalls} = $allArtistCalls;
	$symbolDefs{MediaTitle} = texString($mediaTitle->text);
	$symbolDefs{MediaSubtitle} = texString($mediaSubtitle->text);
	
	updateSymbolPreview();
}

sub updateMediaTitles
{
	$symbolDefs{MediaTitle} = texString($mediaTitle->text);
	$symbolDefs{MediaSubtitle} = texString($mediaSubtitle->text);
	updateSymbolPreview();
}

my %symbolDefListItems; # keys are symbol names, values are Qt::ListViewItem objects
sub updateSymbolPreview
{
	my $symbolName;
	my $symbolDefinition;
	while( ($symbolName, $symbolDefinition) = each %symbolDefs ) {
		my $symbolDefinitionView = $symbolDefinition;
		$symbolDefinitionView =~ s/\n/  /g;
		if( defined($symbolDefListItems{$symbolName}) ) {
			$symbolDefListItems{$symbolName}->setText( 1, $symbolDefinitionView );
		} else {
			# create new Qt::ListViewItem during initial run
			$symbolDefListItems{$symbolName} = Qt::ListViewItem( $symbolPreviewList, '$' . $symbolName, $symbolDefinitionView );
		}
	}
}

sub createLabelClicked
{
	# prerequisite is that %symbolDefs is already defined.

	my $currentdir = getcwd();
	my $tempdir = '/tmp';
	chdir( $tempdir );
	my $outputfile = 'playlist-cdcover-output.tex';
	
	unless( open (OUT, "> " . $outputfile) )
	{
		Qt::MessageBox::critical(this, "IO Error", "can't write to " . $outputfile . "\n", Qt::MessageBox::Abort(),  Qt::MessageBox::NoButton());
		chdir($currentdir);
		return;
	}
	unless( open (TEMPLATE, "< " . $templateFilename->text  ) )
	{
		Qt::MessageBox::critical(this, "IO Error", "can't read file: " . $templateFilename->text . "\n" .
		"Please select a TeX Template File first.", Qt::MessageBox::Abort(),  Qt::MessageBox::NoButton());
		chdir($currentdir);
		close OUT;
		return;
	}
	
	# copy TEMPLATE to OUT while replacing variables
	while( <TEMPLATE> ) {
		my $line = $_;
		$line =~ s/\$(\w+)/$symbolDefs{$1}/g;
		print OUT $line;
	}
	
	close( OUT );

	# now run tex to create a dvi file, and dvips to create a postscript file, and kghostview to
	# display the file - it can be printed then.
	# It is also checked that the modification times of output files are appropriate.
	
	my $tex_date = stat("playlist-cdcover-output.tex")->mtime;
	#print $tex_date . "\n";
	
	system("xterm -e latex playlist-cdcover-output.tex");
	my $dvi_date = stat("playlist-cdcover-output.dvi")->mtime;
	#print $dvi_date . "\n";
	
	if( defined($dvi_date) && $tex_date <= $dvi_date ) {
		system("dvips playlist-cdcover-output.dvi -f >playlist-cdcover-output.ps");
		my $ps_date = stat("playlist-cdcover-output.ps")->mtime;
		#print $ps_date . "\n";
	
		if( defined($ps_date) && $dvi_date <= $ps_date ) {
			system("kghostview --seascape playlist-cdcover-output.ps &");
		} else {
			Qt::MessageBox::critical(this, "dvips error", "The output file " . $tempdir . "/playlist-cdcover-output.ps" . " was not created as expected.", Qt::MessageBox::Abort(),  Qt::MessageBox::NoButton());
		}
	} else {
		# usually, TeX creates dvi output even if the input file is incorrect, but we have
		# this message just in case.
		Qt::MessageBox::critical(this, "tex error", "The output file " . $tempdir . "/playlist-cdcover-output.dvi" . " was not created as expected.\nThis may be caused by a faulty template.\nCheck TeX log.", Qt::MessageBox::Abort(),  Qt::MessageBox::NoButton());
	}
	
	chdir($currentdir);
}

sub displayLogClicked()
{
	# TODO: look for a different editor if kwrite not installed?
	system("kwrite /tmp/playlist-cdcover-output.log &");
}
sub displayTexfileClicked()
{
	# TODO: look for a different editor if kwrite not installed?
	system("kwrite /tmp/playlist-cdcover-output.tex &");
}



my %isoTeXmapping;

sub iso2tex {
	my $in = $_[0];
	my $out;
	my $i;
	for( $i = 0; $i < length($in); $i++ ) {
		my $c = substr( $in, $i, 1 );
		my $o = ord($c);
        if( exists($isoTeXmapping{$o}) ) {
			$out = $out . $isoTeXmapping{$o};
			#print $c . ',' . $o . ', tex: ' .  $isoTeXmapping{$o} . "\n";
		} else {
			$out = $out . $c;
			#print $c . ',' . $o . "\n";
		}
	}
	return $out;
}


## A common table of extended ISO8859 characters for translation
sub read_iso_table {
#Description                          CharCode     HTML name   TeX name  
#===================================  =========    =========   ========
my $iso8859_table = <<'END_OF_TABLE';
ampersand                                  38    &amp;       \&
non-breaking space                        160    &nbsp;      ~
inverted exclamation                      161    &iexcl;     !'
cent sign                                 162    &cent;
pound sterling                            163    &pound;     \pounds
general currency sign                     164    &curren;
yen sign                                  165    &yen;
broken vertical bar                       166    &brvbar;
section sign                              167    &sect;      \S
umlaut (dieresis)                         168    &uml;       \"{}
copyright                                 169    &copy;      \copyright
feminine ordinal                          170    &ordf;      $^a$
left angle quote, guillemotleft           171    &laquo;     \guillemotleft
not sign                                  172    &not;
soft hyphen                               173    &shy;
registered trademark                      174    &reg;       \textregistered
macron accent                             175    &macr;
degree sign                               176    &deg;       $^o$
plus or minus                             177    &plusmn;    $\pm$
superscript two                           178    &sup2;      $^2$
superscript three                         179    &sup3;      $^3$
acute accent                              180    &acute;     \'{}
micro sign                                181    &micro;     $\mu$
paragraph sign                            182    &para;      \P
middle dot                                183    &middot;    $\cdot$
cedilla                                   184    &cedil;     \c{}
superscript one                           185    &sup1;      $^1$
masculine ordinal                         186    &ordm;      $^o$
right angle quote, guillemotright         187    &raquo;     \guillemotright
fraction one-fourth                       188    &frac14;    $\frac14$
fraction one-half                         189    &frac12;    $\frac12$
fraction three-fourths                    190    &frac34;    $\frac34$
inverted question mark                    191    &iquest;    ?'
capital A, grave accent                   192    &Agrave;    \`{A}
capital A, acute accent                   193    &Aacute;    \'{A}
capital A, circumflex accent              194    &Acirc;     \^{A}
capital A, tilde                          195    &Atilde;    \~{A}
capital A, dieresis or umlaut mark        196    &Auml;      \"{A}
capital A, ring                           197    &Aring;     {\AA}
capital AE diphthong (ligature)           198    &AElig;     {\AE}
capital C, cedilla                        199    &Ccedil;    \c{C}
capital E, grave accent                   200    &Egrave;    \`{E}
capital E, acute accent                   201    &Eacute;    \'{E}
capital E, circumflex accent              202    &Ecirc;     \^{E}
capital E, dieresis or umlaut mark        203    &Euml;      \"{E}
capital I, grave accent                   204    &Igrave;    \`{I}
capital I, acute accent                   205    &Iacute;    \'{I}
capital I, circumflex accent              206    &Icirc;     \^{I}
capital I, dieresis or umlaut mark        207    &Iuml;      \"{I}
capital Eth, Icelandic                    208    &ETH;
capital N, tilde                          209    &Ntilde;    \~{N}
capital O, grave accent                   210    &Ograve;    \`{O}
capital O, acute accent                   211    &Oacute;    \'{O}
capital O, circumflex accent              212    &Ocirc;     \^{O}
capital O, tilde                          213    &Otilde;    \~{O}
capital O, dieresis or umlaut mark        214    &Ouml;      \"{O}
multiply sign                             215    &times;     $\times$
capital O, slash                          216    &Oslash;    {\O}
capital U, grave accent                   217    &Ugrave;    \`{U}
capital U, acute accent                   218    &Uacute;    \'{U}
capital U, circumflex accent              219    &Ucirc;     \^{U}
capital U, dieresis or umlaut mark        220    &Uuml;      \"{A}
capital Y, acute accent                   221    &Yacute;    \'{Y}
capital THORN, Icelandic                  222    &THORN;     {\TH}
small sharp s, German (sz ligature)       223    &szlig;     {\ss}
small a, grave accent                     224    &agrave;    \`{a}
small a, acute accent                     225    &aacute;    \'{a}
small a, circumflex accent                226    &acirc;     \^{a}
small a, tilde                            227    &atilde;    \~{a}
small a, dieresis or umlaut mark          228    &auml;      \"{a}
small a, ring                             229    &aring;     {\aa}
small ae diphthong (ligature)             230    &aelig;     {\ae}
small c, cedilla                          231    &ccedil;    \c{c}
small e, grave accent                     232    &egrave;    \`{e}
small e, acute accent                     233    &eacute;    \'{e}
small e, circumflex accent                234    &ecirc;     \^{e}
small e, dieresis or umlaut mark          235    &euml;      \"{e}
small i, grave accent                     236    &igrave;    \`{i}
small i, acute accent                     237    &iacute;    \'{i}
small i, circumflex accent                238    &icirc;     \^{i}
small i, dieresis or umlaut mark          239    &iuml;      \"{i}
small eth, Icelandic                      240    &eth;
small n, tilde                            241    &ntilde;    \~{n}
small o, grave accent                     242    &ograve;    \`{o}
small o, acute accent                     243    &oacute;    \'{o}
small o, circumflex accent                244    &ocirc;     \^{o}
small o, tilde                            245    &otilde;    \~{o}
small o, dieresis or umlaut mark          246    &ouml;      \"{o}
division sign                             247    &divide;    $\divide$
small o, slash                            248    &oslash;    {\o}
small u, grave accent                     249    &ugrave;    \`{u}
small u, acute accent                     250    &uacute;    \'{u}
small u, circumflex accent                251    &ucirc;     \^{u}
small u, dieresis or umlaut mark          252    &uuml;      \"{u}
small y, acute accent                     253    &yacute;    \'{y}
small thorn, Icelandic                    254    &thorn;     {\th}
small y, dieresis or umlaut mark          255    &yuml;      \"{y}
END_OF_TABLE
;
# Thanks to Michael Friendly for posting the table on usenet.
	my @entries = split("\n", $iso8859_table);
	foreach (@entries) {
		my ($desc, $code, $html, $tex) = split(/\s\s+/);
		$isoTeXmapping{$code} = $tex;
		#print $code . '=' . $tex . "\n";
	}
}



package main;
use Qt;
use MainWindow;
my $a = Qt::Application(\@ARGV);
my $mainwindow = MainWindow;
$a->setMainWidget($mainwindow);
$mainwindow->show;
exit $a->exec;
